package com.zd.airbottle.service;

import java.util.List;
import com.zd.airbottle.domain.QpTransportUserCredentials;

/**
 * 运输人员证件Service接口
 * 
 * @author zd
 * @date 2022-05-12
 */
public interface IQpTransportUserCredentialsService 
{
    /**
     * 查询运输人员证件
     * 
     * @param id 运输人员证件主键
     * @return 运输人员证件
     */
    QpTransportUserCredentials selectQpTransportUserCredentialsById(Long id);

    /**
     * 查询运输人员证件列表
     * 
     * @param qpTransportUserCredentials 运输人员证件
     * @return 运输人员证件集合
     */
    List<QpTransportUserCredentials> selectQpTransportUserCredentialsList(QpTransportUserCredentials qpTransportUserCredentials);

    /**
     * 新增运输人员证件
     * 
     * @param qpTransportUserCredentials 运输人员证件
     * @return 结果
     */
    int insertQpTransportUserCredentials(QpTransportUserCredentials qpTransportUserCredentials);

    /**
     * 修改运输人员证件
     * 
     * @param qpTransportUserCredentials 运输人员证件
     * @return 结果
     */
    int updateQpTransportUserCredentials(QpTransportUserCredentials qpTransportUserCredentials);

    /**
     * 批量删除运输人员证件
     * 
     * @param ids 需要删除的运输人员证件主键集合
     * @return 结果
     */
    int deleteQpTransportUserCredentialsByIds(Long[] ids);

    /**
     * 删除运输人员证件信息
     * 
     * @param id 运输人员证件主键
     * @return 结果
     */
    int deleteQpTransportUserCredentialsById(Long id);

    /**
     * 根据人员ID查询运输人员证件
     *
     * @param userId 运输人员主键
     * @return 运输人员证件
     */
    QpTransportUserCredentials getUserCredentialsByUserId(Long userId);

    /**
     * 根据人员ID集合查询运输人员证件
     *
     * @param userIds 运输人员主键集合
     * @return 运输人员证件
     */
    List<QpTransportUserCredentials> getUserCredentialsByUserIds(List<Long> userIds);
}
